//---------------------------------------------------------------------------
#ifndef DirectoryViewerH
#define DirectoryViewerH
//---------------------------------------------------------------------------
#include <vcl\SysUtils.hpp>
#include <vcl\Controls.hpp>
#include <vcl\Classes.hpp>
#include <vcl\Forms.hpp>
#include <vcl\ComCtrls.hpp>
//---------------------------------------------------------------------------

enum TOpcionDirectoryViewer { dvVerPadre, dvVerDirectorioActual, dvVerSubDirectorios, dvVerArchivos, dvVerArchivosOcultos, dvVerArchivosSistema, dvVerArchivosSoloLectura, dvVerArchivosTemporales, dvVerTodo };

typedef Set <TOpcionDirectoryViewer, dvVerPadre, dvVerTodo>  TOpcionesDirectoryViewer;

enum TInformacionFicheroDirectoryViewer { ifNombre = 0, ifLongitud = 1, ifTipo = 2, ifModificado = 3, ifCreacion = 4, ifUltimoAcceso = 5, ifAtributos = 6, ifNombreDos = 7, ifExtensionDos = 8, ifRuta = 9, ifVerTodas = 10 };

typedef Set<TInformacionFicheroDirectoryViewer, ifNombre, ifVerTodas>  TInformacionFicherosDirectoryViewer;

struct ParametrosOrdenacionDirectoryViewer {
	TInformacionFicheroDirectoryViewer IdColumna;
    bool Alfabetica;
};

typedef void __fastcall (__closure * TDirectoryChangingEvent) (System::TObject* Sender, const AnsiString ActualDir, const AnsiString NewDir, bool & AllowChange);

class TDirectoryViewer : public TCustomListView
{
private:
	void __fastcall ProcesarLVMGetItem (TMessage & mensaje);
    void __fastcall ProcesarCNNotify (TMessage & mensaje);
	virtual void __fastcall WndProc(Messages::TMessage & Message);
protected:
	char Path [1024];
    AnsiString FMascaraNombreArchivo;
    AnsiString FMascaraExtensionArchivo;
	bool LlamarHeredado;
	SHFILEINFO InformacionDirectorios;
    AnsiString FDirectorioActual;
    TImageList * ListaIconosGrandes;
    TImageList * ListaIconosMini;
    bool FExploracionAutomatica;
    bool FFastLoad;
    int FTotalNumberOfFiles;
    int FTotalNumberOfFilesDisplayed;
    TOpcionesDirectoryViewer FOpciones;
	TInformacionFicherosDirectoryViewer FColumnas;
    struct ParametrosOrdenacionDirectoryViewer ParametrosOrdenacion;
    int PunterosColumnas [ifVerTodas];
    TNotifyEvent FOnDblClick;
    TKeyPressEvent FOnKeyPress;
    TNotifyEvent FOnDirectoryChange;
	TDirectoryChangingEvent FOnDirectoryChanging;
    TNotifyEvent FOnDisplayChange;
    AnsiString IdentificadorDirectorioPadre;
    AnsiString IdentificadorDirectorioActual;
    TViewStyle FEstiloVisible;
    void __fastcall AsignarDirectorioActual (AnsiString directorio);
    void __fastcall AsignarMascara (AnsiString mascara);
    AnsiString __fastcall ObtenerMascara (void);
    void __fastcall AsignarOpciones (TOpcionesDirectoryViewer opciones);
    void __fastcall AsignarColumnas (TInformacionFicherosDirectoryViewer columnas);
	void __fastcall AsignarEstiloVisible (TViewStyle estilo);
    void __fastcall InicializarColumnas (void);
	void __fastcall ObtenerDirectorioActual (void);
	void __fastcall InsertarArchivo (const WIN32_FIND_DATA & atributos);
    AnsiString __fastcall ObtenerStringFechaArchivo (const FILETIME & fecha);
    AnsiString __fastcall ObtenerStringAtributos (DWORD atributos);
    virtual void __fastcall DblClick (void);
    virtual void __fastcall KeyPress (char & tecla);
	virtual void __fastcall ColClick (TListColumn * columna);
	void __fastcall CompararItems (TObject * yo, TListItem * i1, TListItem * i2, int datos, int & resultado);
	char * __fastcall ObtenerExtensionArchivo (char * archivo, bool separar);
    int __fastcall DondeEmpiezaExtensionArchivo (char * archivo);
	bool __fastcall ArrayCumpleMascara (char * texto, char * mascara);
	bool __fastcall ArchivoCumpleMascara (char * fichero);
public:
	__fastcall TDirectoryViewer (TComponent* Owner);
	__fastcall ~TDirectoryViewer (void);
    int __fastcall OrdenarColumna (TListItem * i1, TListItem * i2);
__published:
  	__property AnsiString DirectorioActual = { read = FDirectorioActual, write = AsignarDirectorioActual };
  	__property AnsiString Mascara = { read = ObtenerMascara, write = AsignarMascara };
    __property bool ExploracionAutomatica = { read = FExploracionAutomatica, write = FExploracionAutomatica };
	__property TOpcionesDirectoryViewer Opciones = { read = FOpciones, write = AsignarOpciones };
    __property TInformacionFicherosDirectoryViewer Columnas = { read = FColumnas, write = AsignarColumnas };
    __property TNotifyEvent OnDblClick = { read = FOnDblClick, write = FOnDblClick };
    __property TNotifyEvent OnDirectoryChange = { read = FOnDirectoryChange, write = FOnDirectoryChange };
	__property TDirectoryChangingEvent OnDirectoryChanging = { read = FOnDirectoryChanging, write = FOnDirectoryChanging };
    __property TNotifyEvent OnDisplayChange = { read = FOnDisplayChange, write = FOnDisplayChange };
	__property bool FastLoad = { read = FFastLoad, write = FFastLoad };
    __property int TotalNumberOfFiles = { read = FTotalNumberOfFiles };
    __property int TotalNumberOfFilesDisplayed = { read = FTotalNumberOfFilesDisplayed };
	__property Align;
	__property BorderStyle;
	__property Color;
	__property OnClick;
	__property Ctl3D;
	__property DragMode;
	__property ReadOnly;
	__property Font;
	__property HideSelection;
	__property IconOptions;
	__property Items;
	__property AllocBy;
	__property MultiSelect;
	__property OnEnter;
	__property OnExit;
	__property OnDragDrop;
	__property OnDragOver;
	__property DragCursor;
	__property OnStartDrag;
	__property OnEndDrag;
	__property OnMouseDown;
	__property OnMouseMove;
	__property OnMouseUp;
	__property ParentShowHint;
	__property ShowHint;
	__property PopupMenu;
	__property SortType;
	__property TabOrder;
	__property TabStop;
	__property TViewStyle ViewStyle = { read = FEstiloVisible, write = AsignarEstiloVisible };
	__property Visible;
	__property OnKeyDown;
	__property TKeyPressEvent OnKeyPress = { read = FOnKeyPress, write = FOnKeyPress };
	__property OnKeyUp;
};
//---------------------------------------------------------------------------
#endif
